package migrations

import (
	"context"

	"github.com/uptrace/bun"
)

//nolint:lll
func init() {
	migrations.MustRegister(func(ctx context.Context, db *bun.DB) error {
		_, err := db.ExecContext(ctx, `
create table if not exists servers
(
    key                         varchar(100)                                       not null
        primary key,
    url                         varchar(255)                                       not null,
    open                        boolean                  default false,
    special                     boolean                  default false,
    num_players                 bigint                   default 0,
    num_tribes                  bigint                   default 0,
    num_villages                bigint                   default 0,
    config                      jsonb,
    building_info               jsonb,
    unit_info                   jsonb,
    created_at                  timestamp with time zone default CURRENT_TIMESTAMP not null,
    player_data_updated_at      timestamp with time zone,
    tribe_data_updated_at       timestamp with time zone,
    village_data_updated_at     timestamp with time zone,
    version_code                varchar(6)                                         not null
        references versions,
    player_snapshots_created_at timestamp with time zone,
    tribe_snapshots_created_at  timestamp with time zone,
    ennoblement_data_updated_at timestamp with time zone,
    num_player_villages         bigint                   default 0,
    num_barbarian_villages      bigint                   default 0,
    num_bonus_villages          bigint                   default 0
);

insert into servers (key, url, open, special, num_players, num_tribes, num_villages, config, building_info, unit_info, created_at, player_data_updated_at, tribe_data_updated_at, village_data_updated_at, version_code, player_snapshots_created_at, tribe_snapshots_created_at, ennoblement_data_updated_at, num_player_villages, num_barbarian_villages, num_bonus_villages)
values  ('pls1', 'https://pls1.plemiona.pl', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'pl', null, null, null, 0, 0, 0),
        ('uks1', 'https://uks1.tribalwars.co.uk', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'uk', null, null, null, 0, 0, 0),
        ('hus1', 'https://hus1.klanhaboru.hu', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'hu', null, null, null, 0, 0, 0),
        ('its1', 'https://its1.tribals.it', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'it', null, null, null, 0, 0, 0),
        ('frs1', 'https://frs1.guerretribale.fr', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'fr', null, null, null, 0, 0, 0),
        ('uss1', 'https://uss1.tribalwars.us', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'us', null, null, null, 0, 0, 0),
        ('nls1', 'https://nls1.tribalwars.nl', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'nl', null, null, null, 0, 0, 0),
        ('ess1', 'https://ess1.guerrastribales.es', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'es', null, null, null, 0, 0, 0),
        ('ros1', 'https://ros1.triburile.ro', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'ro', null, null, null, 0, 0, 0),
        ('grs1', 'https://grs1.fyletikesmaxes.gr', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'gr', null, null, null, 0, 0, 0),
        ('brs1', 'https://brs1.tribalwars.com.br', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'br', null, null, null, 0, 0, 0),
        ('trs1', 'https://trs1.klanlar.org', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'tr', null, null, null, 0, 0, 0),
        ('css1', 'https://css1.divokekmeny.cz', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'cs', null, null, null, 0, 0, 0),
        ('chs1', 'https://chs1.staemme.ch', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'ch', null, null, null, 0, 0, 0),
        ('pts1', 'https://pts1.tribalwars.com.pt', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'pt', null, null, null, 0, 0, 0),
        ('ens1', 'https://ens1.tribalwars.net', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'en', null, null, null, 0, 0, 0),
        ('des1', 'https://des1.die-staemme.de', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'de', null, null, null, 0, 0, 0),
        ('sks1', 'https://sks1.divoke-kmene.sk', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'sk', null, null, null, 0, 0, 0),
        ('master', 'https://www.tribalwars.co.uk', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'uk', null, null, null, 0, 0, 0),
        ('rus1', 'https://rus1.voynaplemyon.com', false, true, 0, 0, 0, '{"Win": {"Check": 0}, "Ally": {"Limit": 0, "Levels": 0, "NoHarm": 0, "NoJoin": 0, "NoLeave": 0, "FixedAllies": 0, "NoOtherSupport": 0, "XpRequirements": "", "AllytimeSupport": 0, "PointsMemberCount": 0, "NoOtherSupportType": 0, "WarsAutoacceptDays": 0, "WarsMemberRequirement": 0, "WarsPointsRequirement": 0}, "Game": {"Tech": 0, "Event": 0, "Hauls": 0, "Archer": 0, "Church": 0, "Knight": 0, "HaulsMax": 0, "FakeLimit": 0, "FarmLimit": 0, "HaulsBase": 0, "Stronghold": 0, "Watchtower": 0, "BarbarianRise": 0, "BaseProduction": 0, "KnightNewItems": 0, "SuppressEvents": 0, "BarbarianShrink": 0, "BuildtimeFormula": 0, "BarbarianMaxPoints": 0}, "Misc": {"Tutorial": 0, "KillRanking": 0, "TradeCancelTime": 0}, "Snob": {"Gold": 0, "Rise": 0, "Factor": 0, "MaxDist": 0, "CoinIron": 0, "CoinWood": 0, "CoinStone": 0, "CheapRebuild": 0, "NoBarbConquer": 0}, "Build": {"Destroy": 0}, "Coord": {"Func": 0, "Inner": 0, "MapSize": 0, "BonusNew": 0, "SelectStart": 0, "NobleRestart": 0, "BonusVillages": 0, "EmptyVillages": 0, "StartVillages": 0, "VillageMoveWait": 0}, "Moral": 0, "Night": {"Active": 0, "EndHour": 0, "Duration": 0, "DefFactor": 0, "StartHour": 0}, "Sleep": {"Max": 0, "Min": 0, "Delay": 0, "Active": 0, "MaxAwake": 0, "MinAwake": 0, "WarnTime": 0}, "Speed": 0, "Newbie": {"Days": 0, "Ratio": 0, "RatioDays": 0, "RemoveNewbieVillages": 0}, "Sitter": {"Allow": 0}, "Commands": {"MillisArrival": 0, "CommandCancelTime": 0}, "Buildings": {"CustomFarm": 0, "CustomHide": 0, "CustomIron": 0, "CustomMain": 0, "CustomSnob": 0, "CustomWall": 0, "CustomWood": 0, "CustomPlace": 0, "CustomSmith": 0, "CustomStone": 0, "CustomChurch": 0, "CustomGarage": 0, "CustomMarket": 0, "CustomStable": 0, "CustomStatue": 0, "CustomStorage": 0, "CustomBarracks": 0, "CustomWatchtower": 0}, "UnitSpeed": 0}', '{"Farm": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Hide": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Iron": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Main": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Snob": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wall": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Wood": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Place": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Smith": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stone": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Garage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Market": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Stable": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Statue": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Storage": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Barracks": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}, "Watchtower": {"Pop": 0, "Iron": 0, "Wood": 0, "Stone": 0, "MaxLevel": 0, "MinLevel": 0, "BuildTime": 0, "PopFactor": 0, "IronFactor": 0, "WoodFactor": 0, "StoneFactor": 0, "BuildTimeFactor": 0}}', '{"Axe": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Ram": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Snob": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Heavy": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Light": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Spear": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Sword": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Archer": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Knight": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Marcher": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Militia": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}, "Catapult": {"Pop": 0, "Carry": 0, "Speed": 0, "Attack": 0, "Defense": 0, "BuildTime": 0, "DefenseArcher": 0, "DefenseCavalry": 0}}', CURRENT_TIMESTAMP, null, null, null, 'ru', null, null, null, 0, 0, 0)
ON CONFLICT DO NOTHING;`)
		return err
	}, func(ctx context.Context, db *bun.DB) error {
		_, err := db.ExecContext(ctx, "drop table if exists servers cascade;")
		return err
	})
}
